#include <zlib.h>
#include "t3f/t3f.h"
#include "psf.h"

static void test_config(ALLEGRO_CONFIG * cp)
{
	ALLEGRO_CONFIG_SECTION * s_iter;
	ALLEGRO_CONFIG_ENTRY * e_iter;
	const char * s_val = NULL;
	const char * e_val = NULL;

	s_val = al_get_first_config_section(cp, &s_iter);
	while(s_val)
	{
		e_val = al_get_first_config_entry(cp, s_val, &e_iter);
		while(e_val)
		{
			printf("config section %s entry %s\n", s_val, e_val);
			e_val = al_get_next_config_entry(&e_iter);
		}
		s_val = al_get_next_config_section(&s_iter);
	}
}

OMO_PSF * omo_load_psf_file(const char * fn)
{
	OMO_PSF * psfp = NULL;
	ALLEGRO_FILE * fp = NULL;
	char header[6] = {0};
	char version;

	printf("load3 %s\n", fn);
	psfp = malloc(sizeof(OMO_PSF));
	if(!psfp)
	{
		goto fail;
	}
	memset(psfp, 0, sizeof(OMO_PSF));

	fp = al_fopen(fn, "rb");
	if(!fp)
	{
		goto fail;
	}
	al_fread(fp, header, 3);
	if(header[0] != 'P' || header[1] != 'S' || header[2] != 'F')
	{
		goto fail;
	}
	version = al_fgetc(fp);
	psfp->reserved_size = al_fread32le(fp);
	psfp->compressed_program_size = al_fread32le(fp);
	psfp->crc32 = al_fread32le(fp);
	if(psfp->reserved_size)
	{
		psfp->reserved = malloc(psfp->reserved_size);
		if(!psfp->reserved)
		{
			goto fail;
		}
		al_fread(fp, psfp->reserved, psfp->reserved_size);
	}
	if(psfp->compressed_program_size)
	{
		psfp->compressed_program = malloc(psfp->compressed_program_size);
		if(!psfp->compressed_program)
		{
			goto fail;
		}
		al_fread(fp, psfp->compressed_program, psfp->compressed_program_size);
	}
	if(psfp->crc32 != crc32(crc32(0L, Z_NULL, 0), psfp->compressed_program, psfp->compressed_program_size))
	{
		goto fail;
	}

	/* uncompress program data */
	psfp->program_size = 0x800008;
	psfp->program = (unsigned char *) malloc(psfp->program_size);
	if(!psfp->program)
	{
		goto fail;
	}
	if(uncompress(psfp->program, &psfp->program_size, psfp->compressed_program, psfp->compressed_program_size) != Z_OK)
	{
		goto fail;
	}

	/* read tags */
	al_fread(fp, header, 5);
	if(header[0] == '[' && header[1] == 'T' && header[2] == 'A' && header[3] == 'G' && header[4] == ']')
	{
		psfp->tags = al_load_config_file_f(fp);
	}
	return psfp;

	fail:
	{
		if(fp)
		{
			al_fclose(fp);
		}
		if(psfp)
		{
			omo_destroy_psf(psfp);
		}
		return NULL;
	}
}

void omo_destroy_psf(OMO_PSF * psfp)
{
	if(psfp->tags)
	{
		al_destroy_config(psfp->tags);
	}
	if(psfp->program)
	{
		free(psfp->program);
	}
	if(psfp->compressed_program)
	{
		free(psfp->compressed_program);
	}
	if(psfp->reserved)
	{
		free(psfp->reserved);
	}
	free(psfp);
}

const char * omo_get_psf_tag(OMO_PSF * psfp, const char * name)
{
	if(psfp->tags)
	{
		return al_get_config_value(psfp->tags, "", name);
	}
	return NULL;
}
