#include "t3f/t3f.h"

#include "../codec_handler.h"
#include "psf.h"
#include "port.h"

#define OMO_MAX_PSF_LIBS 10

static int buf_size = 1024;

typedef struct
{
	OMO_PSF * psf;
	ALLEGRO_AUDIO_STREAM * codec_stream;
	ALLEGRO_MUTEX * codec_mutex;
	ALLEGRO_THREAD * codec_thread;
	float volume;
	char info_buffer[256];
	bool paused;
	double buffer_time;
	double current_time;
} CODEC_DATA;

static void * snes9x_update_thread(ALLEGRO_THREAD * thread, void * arg)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)arg;
	ALLEGRO_EVENT_QUEUE * queue;
	short *fragment;

	queue = al_create_event_queue();
	al_register_event_source(queue, al_get_audio_stream_event_source(codec_data->codec_stream));

	while(1)
	{
		ALLEGRO_EVENT event;

		al_wait_for_event(queue, &event);

		if(event.type == ALLEGRO_EVENT_AUDIO_STREAM_FRAGMENT)
		{
			fragment = (short *)al_get_audio_stream_fragment(codec_data->codec_stream);
			if(fragment)
			{
				if(codec_data->paused)
				{
					memset(fragment, 0, sizeof(short) * buf_size * 2);
				}
				else
				{
					al_lock_mutex(codec_data->codec_mutex);
//					port_get_audio(fragment, buf_size);
					codec_data->current_time += codec_data->buffer_time;
					al_unlock_mutex(codec_data->codec_mutex);
				}
				if(!al_set_audio_stream_fragment(codec_data->codec_stream, fragment))
				{
				}
			}
		}
		if(al_get_thread_should_stop(thread))
		{
			break;
		}
	}

	al_destroy_event_queue(queue);

	return NULL;
}

static void * codec_load_file(const char * fn, const char * subfn)
{
	CODEC_DATA * data;
	const char * val;
	const char * val2;
	int i = 0;
	ALLEGRO_PATH * pp;
	char libtag[8];
	uint32_t offset;
	uint32_t size;
	OMO_PSF * psf_lib = NULL;

	printf("load2 %s\n", fn);
/*	if(!port_init())
	{
		goto fail;
	} */
	data = (CODEC_DATA *)malloc(sizeof(CODEC_DATA));
	if(!data)
	{
		goto fail;
	}
	memset(data, 0, sizeof(CODEC_DATA));
	data->psf = omo_load_psf_file(fn);
	if(!data->psf)
	{
		goto fail;
	}
	val = omo_get_psf_tag(data->root_psf, "_lib");
	if(val)
	{
		i++;
		pp = al_create_path(fn);
		if(!pp)
		{
			goto fail;
		}
		while(val)
		{
			al_set_path_filename(pp, val);
			psf_lib = omo_load_psf_file(al_path_cstr(pp, '/'));
			if(!psf_lib)
			{
				goto fail;
			}
			memcpy(&offset, psf_lib->program + 0, sizeof(offset));
			memcpy(&size, psf_lib->program + 4, sizeof(size));
			memcpy(data->psf->program + 8 + offset, psf_lib->program + 8, size);
			sprintf(libtag, "_lib%d", i + 2);
			val = omo_get_psf_tag(data->psf[0], libtag);
			i++;
		}
	}
	return data;

	fail:
	{
		if(data)
		{
			free(data);
		}
	}
	return NULL;
}

static void codec_unload_file(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

//	port_exit();
	free(data);
}

static bool codec_set_loop(void * data, double loop_start, double loop_end, double fade_time, int loop_count)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return false;
}

static bool codec_set_length(void * data, double length)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return false;
}

static bool codec_set_volume(void * data, float volume)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	codec_data->volume = volume;
	if(codec_data->codec_stream)
	{
		al_set_audio_stream_gain(codec_data->codec_stream, codec_data->volume);
	}
	return true;
}

static const char * codec_get_tag(void * data, const char * name)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	if(!strcmp(name, "Album"))
	{
	}
	else if(!strcmp(name, "Artist"))
	{
	}
	else if(!strcmp(name, "Album"))
	{
	}
	else if(!strcmp(name, "Title"))
	{
	}
	else if(!strcmp(name, "Copyright"))
	{
	}
	else if(!strcmp(name, "Comment"))
	{
	}
	else if(!strcmp(name, "Track"))
	{
	}
	else if(!strcmp(name, "Loop Start"))
	{
	}
	else if(!strcmp(name, "Loop End"))
	{
	}
	else if(!strcmp(name, "Fade Time"))
	{
	}
	return NULL;
}

static int codec_get_track_count(void * data, const char * fn)
{
	return 1;
}

static bool codec_play(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	codec_data->codec_stream = al_create_audio_stream(4, buf_size, 44100, ALLEGRO_AUDIO_DEPTH_INT16, ALLEGRO_CHANNEL_CONF_2);
	if(codec_data->codec_stream)
	{
		codec_data->buffer_time = (double)buf_size / 44100.0;
		al_set_audio_stream_gain(codec_data->codec_stream, codec_data->volume);
		al_attach_audio_stream_to_mixer(codec_data->codec_stream, al_get_default_mixer());
		codec_data->codec_thread = al_create_thread(snes9x_update_thread, codec_data);
		if(codec_data->codec_thread)
		{
			codec_data->codec_mutex = al_create_mutex();
			if(codec_data->codec_mutex)
			{
				al_start_thread(codec_data->codec_thread);
				return true;
			}
		}
	}
	return false;
}

static bool codec_pause(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return true;
}

static bool codec_resume(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return true;
}

static void codec_stop(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	al_destroy_thread(codec_data->codec_thread);
	codec_data->codec_thread = NULL;
	al_destroy_mutex(codec_data->codec_mutex);
	codec_data->codec_mutex = NULL;
	al_destroy_audio_stream(codec_data->codec_stream);
	codec_data->codec_stream = NULL;
}

static bool codec_seek(void * data, double pos)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;
	bool ret = false;

	al_lock_mutex(codec_data->codec_mutex);
	al_unlock_mutex(codec_data->codec_mutex);

	return ret;
}

static double codec_get_position(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return 0.0;
}

static double codec_get_length(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return 0.0;
}

/*static float codec_get_position(void)
{
	return al_get_audio_stream_position_secs(t3f_stream);
} */

static bool codec_done_playing(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	return true;
}

static const char * codec_get_info(void * data)
{
	CODEC_DATA * codec_data = (CODEC_DATA *)data;

	sprintf(codec_data->info_buffer, "SNES9x");
	return codec_data->info_buffer;
}

static OMO_CODEC_HANDLER codec_handler;

extern "C" OMO_CODEC_HANDLER * omo_codec_snes9x_get_codec_handler(void)
{
	memset(&codec_handler, 0, sizeof(OMO_CODEC_HANDLER));
	strcpy(codec_handler.id, "SNES9x");
	codec_handler.initialize = NULL;
	codec_handler.load_file = codec_load_file;
	codec_handler.unload_file = codec_unload_file;
	codec_handler.set_loop = codec_set_loop;
	codec_handler.set_length = codec_set_length;
	codec_handler.set_volume = codec_set_volume;
	codec_handler.get_tag = codec_get_tag;
	codec_handler.get_track_count = codec_get_track_count;
	codec_handler.play = codec_play;
	codec_handler.pause = codec_pause;
	codec_handler.resume = codec_resume;
	codec_handler.stop = codec_stop;
	codec_handler.seek = codec_seek;
	codec_handler.get_position = codec_get_position;
	codec_handler.get_length = codec_get_length;
	codec_handler.done_playing = codec_done_playing;
	codec_handler.get_info = codec_get_info;
	codec_handler.types = 0;
	omo_codec_handler_add_type(&codec_handler, ".snsf");
	omo_codec_handler_add_type(&codec_handler, ".minisnsf");
	return &codec_handler;
}
